"""
Phicomm Nas Server Infomation

"""
import time,hashlib

import logging
from datetime import timedelta

import requests
import voluptuous as vol

import homeassistant.helpers.config_validation as cv
from homeassistant.components.sensor import PLATFORM_SCHEMA
from homeassistant.const import (
    CONF_HOST, CONF_PORT, CONF_MAC ,CONF_NAME, CONF_RESOURCES, TEMP_CELSIUS)
from homeassistant.helpers.entity import Entity
from homeassistant.util import Throttle

_LOGGER = logging.getLogger(__name__)
_RESOURCE = 'api/2/all'

DEFAULT_HOST = 'localhost'
DEFAULT_MAC = ''
DEFAULT_NAME = 'PhicommNas'
DEFAULT_PORT = '8080'

MIN_TIME_BETWEEN_UPDATES = timedelta(minutes=1)

SENSOR_TYPES = {
    'device_name': ['Device Name','','mdi:android'],
    'wifi_mac': ['Wifi Mac','','mdi:wifi'],
    'device_space': ['Device Space', '', 'mdi:harddisk'],
    'disk_name': ['Disk Name', '', 'mdi:harddisk'],
    'disk_free': ['Disk free', '', 'mdi:harddisk'],
    'soft_version': ['Soft Version','','mdi:settings'],
#    'memory_use_percent': ['RAM used', '%', 'mdi:memory'],
#    'memory_use': ['RAM used', 'MiB', 'mdi:memory'],
#    'memory_free': ['RAM free', 'MiB', 'mdi:memory'],
#    'swap_use_percent': ['Swap used', '%', 'mdi:memory'],
#    'swap_use': ['Swap used', 'GiB', 'mdi:memory'],
#    'swap_free': ['Swap free', 'GiB', 'mdi:memory'],
#    'processor_load': ['CPU load', '15 min', 'mdi:memory'],
#    'process_running': ['Running', 'Count', 'mdi:memory'],
#    'process_total': ['Total', 'Count', 'mdi:memory'],
#    'process_thread': ['Thread', 'Count', 'mdi:memory'],
#    'process_sleeping': ['Sleeping', 'Count', 'mdi:memory'],
#    'cpu_temp': ['CPU Temp', TEMP_CELSIUS, 'mdi:thermometer'],
#    'docker_active': ['Containers active', '', 'mdi:docker'],
#    'docker_cpu_use': ['Containers CPU used', '%', 'mdi:docker'],
#    'docker_memory_use': ['Containers RAM used', 'MiB', 'mdi:docker'],
}

PLATFORM_SCHEMA = PLATFORM_SCHEMA.extend({
    vol.Required(CONF_HOST, default=DEFAULT_HOST): cv.string,
    vol.Required(CONF_NAME, default=DEFAULT_NAME): cv.string,
    vol.Required(CONF_MAC, default=DEFAULT_MAC): cv.string,
    vol.Optional(CONF_PORT, default=DEFAULT_PORT): cv.port,
    vol.Optional(CONF_RESOURCES):
        vol.All(cv.ensure_list, [vol.In(SENSOR_TYPES)]),
})


def setup_platform(hass, config, add_devices, discovery_info=None):
    """Set up the sensor."""
    name = config.get(CONF_NAME)
    sn = config.get(CONF_MAC)
    host = config.get(CONF_HOST)
    port = config.get(CONF_PORT)
    millis = str(int(round(time.time() * 1000)))
    m = hashlib.md5()
    m1 = "op=getdevinfo&req_time="+millis+"&sn="+sn
    m.update(m1.encode("utf-8"))
    sign = m.hexdigest()
    url = 'http://{}:{}/v1/nas/sysmgr?op=getdevinfo&req_time={}&sign={}'.format(host, port,millis,sign.upper())
    var_conf = config.get(CONF_RESOURCES)

    rest = PhicommNasData(url)
    rest.update()

    dev = []
    for resource in var_conf:
        dev.append(PhicommNasSensor(rest, name, resource))

    add_devices(dev, True)


class PhicommNasSensor(Entity):
    """Implementation of a sensor."""

    def __init__(self, rest, name, sensor_type):
        """Initialize the sensor."""
        self.rest = rest
        self._name = name
        self.type = sensor_type
        self._state = None
        self._unit_of_measurement = SENSOR_TYPES[sensor_type][1]

    @property
    def name(self):
        """Return the name of the sensor."""
        return '{} {}'.format(self._name, SENSOR_TYPES[self.type][0])

    @property
    def icon(self):
        """Icon to use in the frontend, if any."""
        return SENSOR_TYPES[self.type][2]

    @property
    def unit_of_measurement(self):
        """Return the unit the value is expressed in."""
        return self._unit_of_measurement

    @property
    def available(self):
        """Could the device be accessed during the last update call."""
        return self.rest.data is not None

    @property
    def state(self):
        """Return the state of the resources."""
        return self._state

    def update(self):
        """Get the latest data from REST API."""
        self.rest.update()
        value = self.rest.data

        if value is not None:
            if self.type == 'device_name':
                self._state = value['name']
            elif self.type == 'wifi_mac':
                self._state = value['wifimac']
            elif self.type == 'device_space':
                self._state = value['devicespace']
            elif self.type == 'soft_version':
                self._state = value['softversion']
            elif self.type == 'disk_name':
                self._state = value['mDiskList'][0]['diskname']
            elif self.type == 'disk_free':
                self._state = value['mDiskList'][0]['diskspace']

class PhicommNasData:
    """The class for handling the data retrieval."""

    def __init__(self, resource):
        """Initialize the data object."""
        self._resource = resource
        self.data = {}

    @Throttle(MIN_TIME_BETWEEN_UPDATES)
    def update(self):
        """Get the latest data from the REST API."""
        try:
            response = requests.get(self.
_resource, timeout=10)
            response.encoding = "utf-8"
            self.data = response.json()
        except requests.exceptions.ConnectionError:
            _LOGGER.error("Connection error: %s", self._resource)
            self.data = None
